@@ -86,6 +86,23 @@ let split c s =
     with Not_found -> [ suffix s n ]
   in if s="" then [] else split_from 0 ;;
 
+
+let string_scale_size font size s = 
+  let context = Gdk.Screen.get_pango_context () in
+  let font_description = Pango.Font.from_string font in
+  Pango.Font.modify font_description
+    ~size:(int_of_float (size *. (float Pango.scale)))
+    ();
+  Pango.Context.set_font_description context font_description;
+  let layout = Pango.Layout.create context in
+  Pango.Layout.set_text layout s;
+  let width, height = Pango.Layout.get_pixel_size layout in
+  let width = float width in
+  let linear_width = size*. (float (String.length s)) in
+  size*.width/.linear_width,
+  float height
+
+
 (* HSV TO RGB CONVERSION *)
 
 (* If color string in hsv format, convert to hex *)
@@ -250,15 +267,17 @@ let parse_ellipse constr state =
   let h = get_float state in
   constr (pos, w, h)
 
+let invert_y_pos (x,y) = (x,-.y)
+  
 let parse_filled_ellipse =
-  parse_ellipse (fun (p,w,h) -> Filled_ellipse (p,w,h))
+  parse_ellipse (fun (p,w,h) -> Filled_ellipse (invert_y_pos p,w,h))
 
 let parse_unfilled_ellipse =
-  parse_ellipse (fun (p,w,h) -> Unfilled_ellipse (p,w,h))
+  parse_ellipse (fun (p,w,h) -> Unfilled_ellipse (invert_y_pos p,w,h))
 
 let parse_points state =
   let n = get_int state in
-  Array.init n (fun _ -> get_pos state)
+  Array.init n (fun _ -> invert_y_pos (get_pos state))
 
 let parse_filled_polygon state =
   Filled_polygon (parse_points state)
@@ -276,7 +295,7 @@ let parse_filled_bspline state =
   Filled_bspline (parse_points state)
 
 let parse_text state =
-  let pos = get_pos state in
+  let pos = invert_y_pos (get_pos state) in
   let anchor = get_anchor state in
   let width = get_float state in
   let str = parse_bytes state in
