@@ -25,7 +25,7 @@
 
 open Format
 open Pervasives (* for compatibility with ocaml 3.12.0+dev17
-		   (incoming ocaml3.12) *)
+                   (incoming ocaml3.12) *)
 
 (***************************************************************************)
 (** {2 Common stuff} *)
@@ -64,6 +64,12 @@ let fprint_string_user ppf s =
 (*  let s = String.escaped s in*)
     fprintf ppf "\"%s\"" s
 
+let fprint_htmlstring_user ppf s = fprintf ppf "<%s>" s
+
+let fprint_square_not_empty printer ppf = function
+  | [] -> ()
+  | l -> fprintf ppf " [%a]" printer l
+
 type arrow_style =
   [ `None | `Normal | `Inv | `Dot | `Odot | `Invdot | `Invodot ]
 
@@ -80,6 +86,14 @@ let fprint_dir ppf = function
     `TopToBottom -> fprintf ppf "TB"
   | `LeftToRight -> fprintf ppf "LR"
 
+type symbseq =
+  | COMMA
+  | SEMI
+
+let fprint_symbseq ppf = function
+  | COMMA -> pp_print_string ppf ","
+  | SEMI  -> pp_print_string ppf ";"
+
 (** The [ATTRIBUTES] module type defines the interface for the engines. *)
 module type ATTRIBUTES = sig
 
@@ -93,6 +107,7 @@ module type ATTRIBUTES = sig
   type subgraph = {
     sg_name : string;            (** Box name. *)
     sg_attributes : vertex list; (** Box attributes. *)
+    sg_parent : string option;   (** Nested subgraphs. *)
   }
 
 end
@@ -113,6 +128,7 @@ module CommonAttributes = struct
         (** Sets the type size (in points).  Default value is [14]. *)
     | `Label of string
         (** Caption for graph drawing. *)
+    | `HtmlLabel of string
     | `Orientation of [ `Portrait | `Landscape ]
         (** Sets the page orientation.  Default value is [`Portrait]. *)
     | `Page of float * float
@@ -146,24 +162,44 @@ module CommonAttributes = struct
     | `Label of string
         (** Sets the label printed in the node. The string may include escaped
             newlines [\n], [\l], or [\r] for center, left, and right justified
-	    lines.
+            lines.
             Record labels may contain recursive box lists delimited by { | }.
-	*)
+        *)
+    | `HtmlLabel of string
     | `Orientation of float
         (** Node rotation angle, in degrees.  Default value is [0.0]. *)
+    | `Penwidth of float
+        (** Width of the pen (in points) used to draw the border of the node.
+            Default value is [1.0]. *)
     | `Peripheries of int
         (** Sets  the  number  of periphery lines drawn around the polygon. *)
     | `Regular of bool
         (** If [true], then the polygon is made regular, i.e. symmetric about
-	    the x and y axis, otherwise  the polygon   takes   on   the  aspect
-	    ratio of the label.  Default value is [false]. *)
+            the x and y axis, otherwise  the polygon   takes   on   the  aspect
+            ratio of the label.  Default value is [false]. *)
     | `Shape of
         [`Ellipse | `Box | `Circle | `Doublecircle | `Diamond
-        | `Plaintext | `Record | `Polygon of int * float]
+         | `Oval | `Egg | `Triangle | `Invtriangle
+         | `Trapezium | `Invtrapezium
+         | `House | `Invhouse
+         | `Parallelogram | `Doubleoctagon | `Tripleoctagon
+         | `Mdiamond | `Mcircle | `Msquare
+         | `Star | `Underline
+         | `Note | `Tab | `Folder
+         | `Box3d | `Component  | `Promoter
+         | `Cds
+         | `Terminator | `Utr | `Primersite
+         | `Restrictionsite
+         | `Fivepoverhang | `Threepoverhang | `Noverhang
+         | `Assembly | `Signature | `Insulator | `Ribosite | `Rnastab
+         | `Proteasesite | `Proteinstab | `Rpromoter | `Rarrow
+         | `Larrow | `Lpromoter
+         | `Plaintext | `Record | `Polygon of int * float]
         (** Sets the shape of the node.  Default value is [`Ellipse].
             [`Polygon (i, f)] draws a polygon with [n] sides and a skewing
             of [f]. *)
-    | `Style of [ `Filled | `Solid | `Dashed | `Dotted | `Bold | `Invis ]
+    | `Style of
+        [ `Rounded | `Filled | `Solid | `Dashed | `Dotted | `Bold | `Invis ]
         (** Sets the layout style of the node.  Several styles may be combined
             simultaneously. *)
     | `Width of float
@@ -185,13 +221,14 @@ module CommonAttributes = struct
         (** Sets the label font color.  Default value is [black]. *)
     | `Fontname of string
         (** Sets the label font family name.  Default value is
-	    ["Times-Roman"]. *)
+            ["Times-Roman"]. *)
     | `Fontsize of int
         (** Sets the label type size (in points).  Default value is [14]. *)
     | `Label of string
         (** Sets the label to be attached to the edge.  The string may include
-	    escaped newlines [\n], [\l], or [\r] for centered, left, or right
-	    justified lines. *)
+            escaped newlines [\n], [\l], or [\r] for centered, left, or right
+            justified lines. *)
+    | `HtmlLabel of string
     | `Labelfontcolor of color
         (** Sets the font color for head and tail labels.  Default value is
             [black]. *)
@@ -201,6 +238,9 @@ module CommonAttributes = struct
     | `Labelfontsize of int
         (** Sets the font size for head and tail labels (in points).
             Default value is [14]. *)
+    | `Penwidth of float
+        (** Width of the pen (in points) used to draw the edge. Default value
+            is [1.0].  *)
     | `Style of [ `Solid | `Dashed | `Dotted | `Bold | `Invis ]
         (** Sets the layout style of the edge.  Several styles may be combined
             simultaneously. *)
@@ -218,6 +258,7 @@ module CommonAttributes = struct
     | `Fontname s -> fprintf ppf "fontname=%a" fprint_string s
     | `Fontsize i -> fprintf ppf "fontsize=%i" i
     | `Label s -> fprintf ppf "label=%a" fprint_string_user s
+    | `HtmlLabel s -> fprintf ppf "label=%a" fprint_htmlstring_user s
     | `Orientation a -> fprintf ppf "orientation=%a" fprint_orientation a
     | `Page (x, y) -> fprintf ppf "page=\"%f,%f\"" x y
     | `Pagedir a -> fprintf ppf "pagedir=%a" fprint_dir a
@@ -232,15 +273,67 @@ module CommonAttributes = struct
     | `Diamond -> fprintf ppf "diamond"
     | `Plaintext -> fprintf ppf "plaintext"
     | `Record -> fprintf ppf "record"
+    | `Egg -> fprintf ppf "egg"
+    | `House -> fprintf ppf "house"
+    | `Invhouse -> fprintf ppf "invhouse"
+    | `Trapezium ->  fprintf ppf "trapezium"
+    | `Invtrapezium ->  fprintf ppf "invtrapezium"
+    | `Triangle -> fprintf ppf "triangle"
+    | `Invtriangle -> fprintf ppf "invtriangle"
+    | `Oval ->  fprintf ppf "oval"
+    | `Assembly -> fprintf ppf "assembly"
+    | `Box3d -> fprintf ppf "box3d"
+    | `Cds -> fprintf ppf "cds"
+    | `Component -> fprintf ppf "component"
+    | `Doubleoctagon -> fprintf ppf "doubleoctagon"
+    | `Fivepoverhang -> fprintf ppf "fivepoverhang"
+    | `Folder -> fprintf ppf "folder"
+    | `Insulator -> fprintf ppf "insulator"
+    | `Larrow -> fprintf ppf "larrow"
+    | `Lpromoter -> fprintf ppf "lpromoter"
+    | `Mcircle -> fprintf ppf "mcircle"
+    | `Mdiamond -> fprintf ppf "mdiamond"
+    | `Msquare -> fprintf ppf " msquare"
+    | `Note -> fprintf ppf "note"
+    | `Noverhang -> fprintf ppf "noverhang"
+    | `Parallelogram -> fprintf ppf "parallelogram"
+    | `Primersite -> fprintf ppf "primersite"
+    | `Promoter -> fprintf ppf "promoter"
+    | `Proteasesite -> fprintf ppf "proteasesite"
+    | `Proteinstab -> fprintf ppf "proteinstab"
+    | `Rarrow -> fprintf ppf "rarrow"
+    | `Restrictionsite -> fprintf ppf "restrictionsite"
+    | `Ribosite -> fprintf ppf "ribosite"
+    | `Rnastab -> fprintf ppf "rnastab"
+    | `Rpromoter -> fprintf ppf "rpromoter"
+    | `Signature -> fprintf ppf "signature"
+    | `Star -> fprintf ppf "star"
+    | `Tab -> fprintf ppf "tab"
+    | `Terminator -> fprintf ppf "terminator"
+    | `Threepoverhang -> fprintf ppf "threepoverhang"
+    | `Tripleoctagon -> fprintf ppf "tripleoctagon"
+    | `Underline -> fprintf ppf "underline"
+    | `Utr -> fprintf ppf "utr"
     | `Polygon (i, f) -> fprintf ppf "polygon, sides=%i, skew=%f" i f
 
-  let fprint_node_style ppf = function
-      `Filled -> fprintf ppf "filled"
-    | `Solid -> fprintf ppf "solid"
-    | `Dashed -> fprintf ppf "dashed"
-    | `Dotted -> fprintf ppf "dotted"
-    | `Bold -> fprintf ppf "bold"
-    | `Invis -> fprintf ppf "invis"
+  let rec fprint_string_list ppf = function
+    | [] -> ()
+    | [hd] -> fprintf ppf "%s" hd
+    | hd :: tl -> fprintf ppf "%s,%a" hd fprint_string_list tl
+
+  let node_style_str = function
+    | `Rounded -> "rounded"
+    | `Filled -> "filled"
+    | `Solid -> "solid"
+    | `Dashed -> "dashed"
+    | `Dotted -> "dotted"
+    | `Bold -> "bold"
+    | `Invis -> "invis"
+
+  let fprint_style_list sep ppf a =
+    fprintf ppf "style=\"%a\"%a@ "
+      fprint_string_list (List.map node_style_str a)
+      fprint_symbseq sep
 
   let fprint_vertex ppf = function
     | `Color a -> fprintf ppf "color=%a" fprint_color a
@@ -251,15 +344,17 @@ module CommonAttributes = struct
     | `Fontsize i -> fprintf ppf "fontsize=%i" i
     | `Height f -> fprintf ppf "height=%f" f
     | `Label s -> fprintf ppf "label=%a" fprint_string_user s
+    | `HtmlLabel s -> fprintf ppf "label=%a" fprint_htmlstring_user s
     | `Orientation f -> fprintf ppf "orientation=%f" f
+    | `Penwidth f -> fprintf ppf "penwidth=%f" f
     | `Peripheries i -> fprintf ppf "peripheries=%i" i
     | `Regular b -> fprintf ppf "regular=%b" b
     | `Shape a -> fprintf ppf "shape=%a" fprint_shape a
-    | `Style a -> fprintf ppf "style=%a" fprint_node_style a
+    | `Style _ -> assert false
     | `Width f -> fprintf ppf "width=%f" f
 
-  let fprint_edge_style =
-    fprint_node_style
+  let edge_style_str =
+    node_style_str
 
   let fprint_arrow_direction ppf = function
       `Forward -> fprintf ppf "forward"
@@ -277,11 +372,46 @@ module CommonAttributes = struct
     | `Fontname s -> fprintf ppf "fontname=%a" fprint_string s
     | `Fontsize i -> fprintf ppf "fontsize=%i" i
     | `Label s -> fprintf ppf "label=%a" fprint_string_user s
+    | `HtmlLabel s -> fprintf ppf "label=%a" fprint_htmlstring_user s
     | `Labelfontcolor a -> fprintf ppf "labelfontcolor=%a" fprint_color a
     | `Labelfontname s -> fprintf ppf "labelfontname=\"%s\"" s
-	(* (String.escaped s) *)
+        (* (String.escaped s) *)
     | `Labelfontsize i -> fprintf ppf "labelfontsize=%i" i
-    | `Style a -> fprintf ppf "style=%a" fprint_edge_style a
+    | `Penwidth f -> fprintf ppf "penwidth=%f" f
+    | `Style _ -> assert false
+
+  let rec filter_style al sl l = match l with
+    | [] -> al, sl
+    | `Style s :: l -> filter_style al (s :: sl) l
+    | a :: l -> filter_style (a :: al) sl l
+
+  (** [fprint_graph_attribute printer ppf list] pretty prints a list of
+      attributes on the formatter [ppf], using the printer [printer] for
+      each attribute.  The list appears between brackets and attributes
+      are speparated by ",".  If the list is empty, nothing is printed. *)
+  let fprint_attributes fprint_style_list fprint_attribute sep ppf list =
+    if list <> [] then begin
+      let list, styles = filter_style [] [] list in
+      let rec fprint_attributes_rec ppf = function
+        | [] -> ()
+        | hd :: tl ->
+            fprintf ppf "%a%a@ "
+                fprint_attribute hd
+                fprint_symbseq sep;
+            fprint_attributes_rec ppf tl
+      in
+      fprintf ppf "@[<hov>%a" fprint_attributes_rec list;
+      if styles <> [] then begin
+        fprint_style_list sep ppf styles
+      end;
+      fprintf ppf "@]"
+    end
+
+    let fprint_vertex_list =
+      fprint_attributes fprint_style_list fprint_vertex
+
+    let fprint_edge_list =
+      fprint_attributes fprint_style_list fprint_edge
 
 end
 
@@ -295,8 +425,8 @@ module type ENGINE = sig
   module Attributes : sig
     include ATTRIBUTES
     val fprint_graph:formatter -> graph -> unit
-    val fprint_vertex: formatter -> vertex -> unit
-    val fprint_edge: formatter -> edge -> unit
+    val fprint_vertex_list: symbseq -> formatter -> vertex list -> unit
+    val fprint_edge_list: symbseq -> formatter -> edge list -> unit
   end
 
   (** The litteral name of the engine. *)
@@ -346,136 +476,118 @@ struct
     try
       f arg
     with
-	Error msg ->
-	  Printf.eprintf "%s: %s failure\n   %s\n"
-	  Sys.argv.(0) EN.name msg;
-	  flush stderr;
-	  exit 2
+        Error msg ->
+          Printf.eprintf "%s: %s failure\n   %s\n"
+          Sys.argv.(0) EN.name msg;
+          flush stderr;
+          exit 2
 
     (** [fprint_graph_attributes ppf list] pretty prints a list of
         graph attributes on the formatter [ppf].  Attributes are separated
         by a ";". *)
     let fprint_graph_attributes ppf list =
        List.iter (function att ->
-	 fprintf ppf "%a;@ " EN.Attributes.fprint_graph att
+         fprintf ppf "%a;@ " EN.Attributes.fprint_graph att
        ) list
 
-   (** [fprint_graph_attribute printer ppf list] pretty prints a list of
-       attributes on the formatter [ppf], using the printer [printer] for
-       each attribute.  The list appears between brackets and attributes
-       are speparated by ",".  If the list is empty, nothing is printed. *)
-    let fprint_attributes fprint_attribute ppf = function
-	[] -> ()
-      | hd :: tl ->
-	  let rec fprint_attributes_rec ppf = function
-	      [] -> ()
-	    | hd' :: tl' ->
-		fprintf ppf ",@ %a%a"
-		  fprint_attribute hd'
-		  fprint_attributes_rec tl'
-	  in
-	  fprintf ppf " [@[<hov>%a%a@]]"
-	    fprint_attribute hd
-	    fprint_attributes_rec tl
-
-    (** [fprint_graph_attributes ppf list] pretty prints a list of
-        node attributes using the format of [fprint_attributes]. *)
-    let fprint_node_attributes ppf list =
-      fprint_attributes EN.Attributes.fprint_vertex ppf list
-
-    (** [fprint_graph_attributes ppf list] pretty prints a list of
-        edge attributes using the format of [fprint_attributes]. *)
-    let fprint_edge_attributes ppf list =
-      fprint_attributes EN.Attributes.fprint_edge ppf list
-
     (** [fprint_graph ppf graph] pretty prints the graph [graph] in
         the CGL language on the formatter [ppf]. *)
     let fprint_graph ppf graph =
-      let subgraphs = Hashtbl.create 7 in
+      let module SG = Map.Make(String) in
+      let subgraphs = ref SG.empty in
 
       (* Printing nodes. *)
 
       let print_nodes ppf =
+        let default_node_attributes = X.default_vertex_attributes graph in
+          if default_node_attributes  <> [] then
+            fprintf ppf "node%a;@ "
+              (fprint_square_not_empty (EN.Attributes.fprint_vertex_list COMMA))
+              default_node_attributes;
 
-	let default_node_attributes = X.default_vertex_attributes graph in
-	if default_node_attributes  <> [] then
-	  fprintf ppf "node%a;@ "
-	    fprint_node_attributes default_node_attributes;
-
-	X.iter_vertex
-          (function node ->
-             begin match X.get_subgraph node with
-             | None -> ()
-             | Some sg ->
-                 try
-                   let (sg,nodes) =
-		     Hashtbl.find subgraphs sg.EN.Attributes.sg_name
-		   in
-                   Hashtbl.replace subgraphs
-		     sg.EN.Attributes.sg_name (sg,node::nodes)
-                 with Not_found ->
-                   Hashtbl.add subgraphs sg.EN.Attributes.sg_name (sg,[node])
-             end;
-	     fprintf ppf "%s%a;@ "
-	       (X.vertex_name node)
-	       fprint_node_attributes (X.vertex_attributes node))
-          graph
-
+            X.iter_vertex
+              (function node ->
+                begin match X.get_subgraph node with
+                | None -> ()
+                | Some sg ->
+                    let (sg, nodes) =
+                      if SG.mem sg.EN.Attributes.sg_name !subgraphs then
+                        SG.find sg.EN.Attributes.sg_name !subgraphs
+                      else
+                        (sg, [])
+                    in
+                      subgraphs := SG.add sg.EN.Attributes.sg_name (sg, node :: nodes) !subgraphs
+                end;
+                fprintf ppf "%s%a;@ "
+                  (X.vertex_name node)
+                  (fprint_square_not_empty (EN.Attributes.fprint_vertex_list COMMA))
+                  (X.vertex_attributes node)
+              )
+              graph
       in
 
       (* Printing subgraphs *)
 
+      let rec print_nested_subgraphs ppf = function
+        | [] ->
+            () (* no more work to do, so terminate *)
+        | name :: worklist ->
+            let sg, nodes = SG.find name !subgraphs in
+            let children = SG.filter (fun n (sg, nodes) -> sg.EN.Attributes.sg_parent = Some name) !subgraphs in
+              fprintf ppf "@[<v 2>subgraph cluster_%s { %a%t@ %t };@]@\n"
+
+              name
+
+              (EN.Attributes.fprint_vertex_list SEMI)
+              sg.EN.Attributes.sg_attributes
+
+              (fun ppf ->
+                (List.iter (fun n -> fprintf ppf "%s;" (X.vertex_name n)) nodes)
+              )
+
+              (fun ppf ->
+                print_nested_subgraphs ppf (List.map fst (SG.bindings children))
+              );
+
+              print_nested_subgraphs ppf worklist
+      in
+
       let print_subgraphs ppf =
-
-        Hashtbl.iter
-          (fun name (sg,nodes) ->
-             fprintf ppf "@[<v 2>subgraph cluster_%s { %t%t };@]@\n"
-               name
-
-               (fun ppf ->
-                  (List.iter
-                     (fun n ->
-			fprintf ppf "%a;@\n" EN.Attributes.fprint_vertex n)
-                     sg.EN.Attributes.sg_attributes))
-
-               (fun ppf ->
-                  (List.iter
-		     (fun n -> fprintf ppf "%s;" (X.vertex_name n))
-                     nodes))
-          )
-          subgraphs
-
+        let root_worklist = SG.filter (fun n (sg, nodes) -> sg.EN.Attributes.sg_parent = None) !subgraphs in
+          print_nested_subgraphs ppf (List.map fst (SG.bindings root_worklist))
       in
 
       (* Printing edges *)
 
       let print_edges ppf =
 
-	let default_edge_attributes = X.default_edge_attributes graph in
-	if default_edge_attributes <> [] then
-	  fprintf ppf "edge%a;@ "
-	    fprint_edge_attributes default_edge_attributes;
+        let default_edge_attributes = X.default_edge_attributes graph in
+        if default_edge_attributes <> [] then
+          fprintf ppf "edge%a;@ "
+            (fprint_square_not_empty (EN.Attributes.fprint_edge_list COMMA))
+            default_edge_attributes;
 
-	X.iter_edges_e (function edge ->
-	                  fprintf ppf "%s %s %s%a;@ "
-	                    (X.vertex_name (X.E.src edge))
-	                    EN.edge_arrow
-	                    (X.vertex_name (X.E.dst edge))
-	                    fprint_edge_attributes (X.edge_attributes edge)
+        X.iter_edges_e (function edge ->
+                          fprintf ppf "%s %s %s%a;@ "
+                            (X.vertex_name (X.E.src edge))
+                            EN.edge_arrow
+                            (X.vertex_name (X.E.dst edge))
+                            (fprint_square_not_empty (EN.Attributes.fprint_edge_list COMMA))
+                            (X.edge_attributes edge)
                        ) graph
 
       in
 
       fprintf ppf "@[<v>%s G {@ @[<v 2>  %a"
-	EN.opening
-	fprint_graph_attributes (X.graph_attributes graph);
+        EN.opening
+        fprint_graph_attributes (X.graph_attributes graph);
       fprintf ppf "%t@ " print_nodes;
       fprintf ppf "%t@ " print_subgraphs;
       fprintf ppf "%t@ " print_edges;
       fprintf ppf "@]}@]"
 
     (** [output_graph oc graph] pretty prints the graph [graph] in the dot
-	language on the channel [oc]. *)
+        language on the channel [oc]. *)
     let output_graph oc graph =
       let ppf = formatter_of_out_channel oc in
       fprint_graph ppf graph;
@@ -529,14 +641,14 @@ module DotAttributes = struct
         (** Sets the minimum separation between ranks. *)
     | `Quantum of float
         (** If not [0.0], node label dimensions will be rounded to integral
-	    multiples of it.  Default value is [0.0]. *)
+            multiples of it.  Default value is [0.0]. *)
     | `Rankdir of [ `TopToBottom | `LeftToRight ]
         (** Direction of rank ordering.  Default value is [`TopToBottom]. *)
     | `Ratio of [ `Float of float | `Fill | `Compress| `Auto ]
         (** Sets the aspect ratio. *)
     | `Samplepoints of int
         (** Number of points used to represent ellipses and circles on output.
-	    Default value is [8]. *)
+            Default value is [8]. *)
     | `Url of string
         (** URL associated with graph (format-dependent). *)
     ]
@@ -566,7 +678,7 @@ module DotAttributes = struct
     | `Url of string
         (** The  default  url  for  image  map  files; in PostScript files,
             the base URL for all relative URLs, as recognized by Acrobat
-	    Distiller 3.0 and up. *)
+            Distiller 3.0 and up. *)
     | `Z of float
         (** z coordinate for VRML output. *)
     ]
@@ -609,10 +721,10 @@ module DotAttributes = struct
         (** Minimum rank distance between head an tail.  Default value is [1]. *)
     | `Samehead of string
         (** Tag for head node; edge heads with the same tag are merged onto the
-	    same port. *)
+            same port. *)
     | `Sametail of string
         (** Tag for tail node; edge tails with the same tag are merged onto the
-	    same port. *)
+            same port. *)
     | `Taillabel of string
         (** Sets the label attached to the tail arrow. *)
     | `Tailport of [ `N | `NE | `E | `SE | `S | `SW | `W | `NW ]
@@ -627,6 +739,7 @@ module DotAttributes = struct
     type subgraph = {
       sg_name : string;
       sg_attributes : vertex list;
+      sg_parent : string option;
     }
 
     (** {4 Pretty-print of attributes} *)
@@ -637,13 +750,13 @@ module DotAttributes = struct
       | hd :: tl -> fprintf ppf "%s,%a" hd fprint_string_list tl
 
     let fprint_ratio ppf = function
-	`Float f -> fprintf ppf "%f" f
+        `Float f -> fprintf ppf "%f" f
       | `Fill -> fprintf ppf "fill"
       | `Compress -> fprintf ppf "compress"
       | `Auto -> fprintf ppf "auto"
 
     let fprint_graph ppf = function
-	#CommonAttributes.graph as att -> CommonAttributes.fprint_graph ppf att
+        #CommonAttributes.graph as att -> CommonAttributes.fprint_graph ppf att
       | `Bgcolor a -> fprintf ppf "bgcolor=%a" fprint_color a
       | `BgcolorWithTransparency a ->
           fprintf ppf "bcolor=%a" fprint_color_with_transparency a
@@ -664,8 +777,8 @@ module DotAttributes = struct
       | `Url s -> fprintf ppf "URL=\"%s\"" s (*(String.escaped s)*)
 
     let fprint_vertex ppf = function
-	#CommonAttributes.vertex as att ->
-	  CommonAttributes.fprint_vertex ppf att
+        #CommonAttributes.vertex as att ->
+          CommonAttributes.fprint_vertex ppf att
       | `Comment s -> fprintf ppf "comment=%a" fprint_string s
       | `Distortion f -> fprintf ppf "distortion=%f" f
       | `Fillcolor a -> fprintf ppf "fillcolor=%a" fprint_color a
@@ -677,7 +790,7 @@ module DotAttributes = struct
       | `Z f -> fprintf ppf "z=%f" f
 
     let fprint_port ppf = function
-	`N -> fprintf ppf "n"
+        `N -> fprintf ppf "n"
       | `NE -> fprintf ppf "ne"
       | `E -> fprintf ppf "e"
       | `SE -> fprintf ppf "se"
@@ -687,7 +800,7 @@ module DotAttributes = struct
       | `NW -> fprintf ppf "nw"
 
     let fprint_edge ppf = function
-	#CommonAttributes.edge as att -> CommonAttributes.fprint_edge ppf att
+        #CommonAttributes.edge as att -> CommonAttributes.fprint_edge ppf att
       | `Arrowhead a -> fprintf ppf "arrowhead=%a" fprint_arrow_style a
       | `Arrowsize f -> fprintf ppf "arrowsize=%f" f
       | `Arrowtail a -> fprintf ppf "arrowtail=%a" fprint_arrow_style a
@@ -708,6 +821,14 @@ module DotAttributes = struct
       | `Tailurl s -> fprintf ppf "tailURL=%a" fprint_string s
       | `Weight i -> fprintf ppf "weight=%i" i
 
+    let fprint_vertex_list =
+      CommonAttributes.fprint_attributes
+        CommonAttributes.fprint_style_list fprint_vertex
+
+    let fprint_edge_list =
+      CommonAttributes.fprint_attributes
+        CommonAttributes.fprint_style_list fprint_edge
+
 end
 
 (** Graph modules with dot attributes *)
@@ -728,17 +849,17 @@ module type GraphWithDotAttrs = sig
 
   val get_subgraph : V.t -> DotAttributes.subgraph option
     (** The box (if exists) which the vertex belongs to. Boxes with same
-	   names are not distinguished and so they should have the same
-	   attributes. *)
+           names are not distinguished and so they should have the same
+           attributes. *)
 end
 
 module Dot =
   MakeEngine (struct
-		module Attributes = DotAttributes
-		let name = "dot"
-		let opening = "digraph"
-		let edge_arrow = "->"
-	      end)
+                module Attributes = DotAttributes
+                let name = "dot"
+                let opening = "digraph"
+                let edge_arrow = "->"
+              end)
 
 (***************************************************************************)
 (** {2 Interface with the neato engine} *)
@@ -758,13 +879,13 @@ module NeatoAttributes = struct
     | `Start of int
         (** Seed for random number generator. *)
     | `Overlap of bool
-	(** Default value is [true]. *)
+        (** Default value is [true]. *)
     | `Spline of bool
-	(** [true] makes edge splines if nodes don't overlap.
-	    Default value is [false]. *)
+        (** [true] makes edge splines if nodes don't overlap.
+            Default value is [false]. *)
     | `Sep of float
-	(** Edge spline separation factor from nodes.  Default value
-	    is [0.0]. *)
+        (** Edge spline separation factor from nodes.  Default value
+            is [0.0]. *)
     ]
 
   (** Attributes of nodes.  They include all common node attributes and
@@ -792,12 +913,13 @@ module NeatoAttributes = struct
     type subgraph = {
       sg_name : string;
       sg_attributes : vertex list;
+      sg_parent : string option;
     }
 
   (** {4 Pretty-print of attributes} *)
 
     let fprint_graph ppf = function
-	#CommonAttributes.graph as att -> CommonAttributes.fprint_graph ppf att
+        #CommonAttributes.graph as att -> CommonAttributes.fprint_graph ppf att
       | `Margin (f1, f2) -> fprintf ppf "margin=\"%f,%f\"" f1 f2
       | `Start i -> fprintf ppf "start=%i" i
       | `Overlap b -> fprintf ppf "overlap=%b" b
@@ -805,25 +927,33 @@ module NeatoAttributes = struct
       | `Sep f -> fprintf ppf "sep=%f" f
 
     let fprint_vertex ppf = function
-	#CommonAttributes.vertex as att ->
-	  CommonAttributes.fprint_vertex ppf att
+        #CommonAttributes.vertex as att ->
+          CommonAttributes.fprint_vertex ppf att
       | `Pos (f1, f2) -> fprintf ppf "pos=\"%f,%f\"" f1 f2
 
     let fprint_edge ppf = function
-	#CommonAttributes.edge as att -> CommonAttributes.fprint_edge ppf att
+        #CommonAttributes.edge as att -> CommonAttributes.fprint_edge ppf att
       | `Id s -> fprintf ppf "id=%a" fprint_string s
       | `Len f -> fprintf ppf "len=%f" f
       | `Weight f -> fprintf ppf "weight=%f" f
 
+    let fprint_vertex_list =
+      CommonAttributes.fprint_attributes
+        CommonAttributes.fprint_style_list fprint_vertex
+
+    let fprint_edge_list =
+      CommonAttributes.fprint_attributes
+        CommonAttributes.fprint_style_list fprint_edge
+
 end
 
 module Neato =
   MakeEngine (struct
-		module Attributes = NeatoAttributes
-		let name = "neato"
-		let opening = "graph"
-		let edge_arrow = "--"
-	      end)
+                module Attributes = NeatoAttributes
+                let name = "neato"
+                let opening = "graph"
+                let edge_arrow = "--"
+              end)
 
 (*
 Local Variables:
