@@ -63,6 +63,7 @@ module type ATTRIBUTES = sig
   type subgraph = {
     sg_name : string;            (** Box name. *)
     sg_attributes : vertex list; (** Box attributes. *)
+    sg_parent : string option;   (** Nested subgraphs. *)
   }
 
 end
@@ -86,8 +87,14 @@ module CommonAttributes : sig
         (** Sets the type size (in points).  Default value is [14]. *)
     | `Label of string
         (** Caption for graph drawing. *)
+    | `HtmlLabel of string
+    (** Caption for graph drawing. In HTML strings, angle brackets must occur in
+        matched pairs, and newlines and other formatting whitespace characters
+        are allowed. In addition, the content must be legal XML, so that the
+        special XML escape sequences for ", &, <, and > may be necessary in
+        order to embed these characters in attribute values or raw text."  *)
     | `Orientation of [ `Portrait | `Landscape ]
-        (** Sets the page orientation.  Default value is [`Portrait]. *)
+    (** Sets the page orientation.  Default value is [`Portrait]. *)
     | `Page of float * float
         (** Sets the PostScript pagination unit, e.g [8.5, 11.0]. *)
     | `Pagedir of [ `TopToBottom | `LeftToRight ]
@@ -103,7 +110,7 @@ module CommonAttributes : sig
   type vertex =
     [ `Color of color
         (** Sets the color of the border of the vertex.
-	  Default value is [black] *)
+          Default value is [black] *)
     | `ColorWithTransparency of color_with_transparency
         (** Sets the color of the border of the vertex with a transparency
             component. Default value is fully opaque [black] *)
@@ -119,28 +126,56 @@ module CommonAttributes : sig
         (** Sets the minimum height.  Default value is [0.5]. *)
     | `Label of string
         (** Sets the label printed in the vertex.
-	    The string may include escaped
+            The string may include escaped
             newlines [\n], [\l], or [\r] for center, left, and right justified
-	    lines.
+            lines.
             Record labels may contain recursive box lists delimited by { | }.
-	*)
+        *)
+    | `HtmlLabel of string
+    (** Like label, in html style. In HTML strings, angle brackets must occur in
+        matched pairs, and newlines and other formatting whitespace characters
+        are allowed. In addition, the content must be legal XML, so that the
+        special XML escape sequences for ", &, <, and > may be necessary in
+        order to embed these characters in attribute values or raw text."  *)
     | `Orientation of float
-        (** Vertex rotation angle, in degrees.  Default value is [0.0]. *)
+    (** Vertex rotation angle, in degrees.  Default value is [0.0]. *)
+    | `Penwidth of float
+        (** Width of the pen (in points) used to draw the border of the node.
+            Default value is [1.0]. *)
     | `Peripheries of int
         (** Sets  the  number  of periphery lines drawn around the polygon. *)
     | `Regular of bool
         (** If [true], then the polygon is made regular, i.e. symmetric about
-	    the x and y axis, otherwise  the polygon   takes   on   the  aspect
-	    ratio of the label.  Default value is [false]. *)
+            the x and y axis, otherwise  the polygon   takes   on   the  aspect
+            ratio of the label.  Default value is [false]. *)
     | `Shape of
         [`Ellipse | `Box | `Circle | `Doublecircle | `Diamond
-        | `Plaintext | `Record | `Polygon of int * float]
-        (** Sets the shape of the vertex.  Default value is [`Ellipse].
+         | `Plaintext | `Record
+         (* Addition through http://www.graphviz.org/doc/info/shapes.html *)
+         | `Oval | `Egg | `Triangle | `Invtriangle
+         | `Trapezium | `Invtrapezium
+         | `House | `Invhouse
+         | `Parallelogram | `Doubleoctagon | `Tripleoctagon
+         | `Mdiamond | `Mcircle | `Msquare
+         | `Star | `Underline
+         | `Note | `Tab | `Folder
+         | `Box3d | `Component  | `Promoter
+         | `Cds
+         | `Terminator | `Utr | `Primersite
+         | `Restrictionsite
+         | `Fivepoverhang | `Threepoverhang | `Noverhang
+         | `Assembly | `Signature | `Insulator | `Ribosite | `Rnastab
+         | `Proteasesite | `Proteinstab | `Rpromoter | `Rarrow
+         | `Larrow | `Lpromoter
+         (* Addition ends here *)
+         | `Polygon of int * float]
+    (** Sets the shape of the vertex.  Default value is [`Ellipse].
             [`Polygon (i, f)] draws a polygon with [n] sides and a skewing
             of [f]. *)
-    | `Style of [ `Filled | `Solid | `Dashed | `Dotted | `Bold | `Invis ]
+    | `Style of
+        [ `Rounded | `Filled | `Solid | `Dashed | `Dotted | `Bold | `Invis ]
         (** Sets the layout style of the vertex.
-	    Several styles may be combined simultaneously. *)
+            Several styles may be combined simultaneously. *)
     | `Width of float
         (** Sets the minimum width.  Default value is [0.75]. *)
     ]
@@ -160,13 +195,19 @@ module CommonAttributes : sig
         (** Sets the label font color.  Default value is [black]. *)
     | `Fontname of string
         (** Sets the label font family name.  Default value is
-	    ["Times-Roman"]. *)
+            ["Times-Roman"]. *)
     | `Fontsize of int
         (** Sets the label type size (in points).  Default value is [14]. *)
     | `Label of string
         (** Sets the label to be attached to the edge.  The string may include
-	    escaped newlines [\n], [\l], or [\r] for centered, left, or right
-	    justified lines. *)
+            escaped newlines [\n], [\l], or [\r] for centered, left, or right
+            justified lines. *)
+    | `HtmlLabel of string
+    (** Like label, in html style. In HTML strings, angle brackets must occur in
+        matched pairs, and newlines and other formatting whitespace characters
+        are allowed. In addition, the content must be legal XML, so that the
+        special XML escape sequences for ", &, <, and > may be necessary in
+        order to embed these characters in attribute values or raw text."  *)
     | `Labelfontcolor of color
         (** Sets the font color for head and tail labels.  Default value is
             [black]. *)
@@ -176,6 +217,9 @@ module CommonAttributes : sig
     | `Labelfontsize of int
         (** Sets the font size for head and tail labels (in points).
             Default value is [14]. *)
+    | `Penwidth of float
+        (** Width of the pen (in points) used to draw the edge. Default value
+            is [1.0].  *)
     | `Style of [ `Solid | `Dashed | `Dotted | `Bold | `Invis ]
         (** Sets the layout style of the edge.  Several styles may be combined
             simultaneously. *)
@@ -227,14 +271,14 @@ module DotAttributes : sig
         (** Sets the minimum separation between ranks. *)
     | `Quantum of float
         (** If not [0.0], node label dimensions will be rounded to integral
-	    multiples of it.  Default value is [0.0]. *)
+            multiples of it.  Default value is [0.0]. *)
     | `Rankdir of [ `TopToBottom | `LeftToRight ]
         (** Direction of rank ordering.  Default value is [`TopToBottom]. *)
     | `Ratio of [ `Float of float | `Fill | `Compress| `Auto ]
         (** Sets the aspect ratio. *)
     | `Samplepoints of int
         (** Number of points used to represent ellipses and circles on output.
-	    Default value is [8]. *)
+            Default value is [8]. *)
     | `Url of string
         (** URL associated with graph (format-dependent). *)
     ]
@@ -263,7 +307,7 @@ module DotAttributes : sig
     | `Url of string
         (** The  default  url  for  image  map  files; in PostScript files,
             the base URL for all relative URLs, as recognized by Acrobat
-	    Distiller 3.0 and up. *)
+            Distiller 3.0 and up. *)
     | `Z of float
         (** z coordinate for VRML output. *)
     ]
@@ -303,13 +347,13 @@ module DotAttributes : sig
         (** Overlay. *)
     | `Minlen of int
         (** Minimum rank distance between head an tail.
-	    Default value is [1]. *)
+            Default value is [1]. *)
     | `Samehead of string
         (** Tag for head node; edge heads with the same tag are merged onto the
-	    same port. *)
+            same port. *)
     | `Sametail of string
         (** Tag for tail node; edge tails with the same tag are merged onto the
-	    same port. *)
+            same port. *)
     | `Taillabel of string
         (** Sets the label attached to the tail arrow. *)
     | `Tailport of [ `N | `NE | `E | `SE | `S | `SW | `W | `NW ]
@@ -325,6 +369,7 @@ module DotAttributes : sig
   type subgraph = {
     sg_name : string;
     sg_attributes : vertex list;
+    sg_parent : string option;
   }
 
 end
@@ -349,8 +394,8 @@ module type GraphWithDotAttrs = sig
 
   val get_subgraph : V.t -> DotAttributes.subgraph option
     (** The box (if exists) which the vertex belongs to. Boxes with same
-	   names are not distinguished and so they should have the same
-	   attributes. *)
+           names are not distinguished and so they should have the same
+           attributes. *)
 
 end
 
@@ -375,8 +420,8 @@ module Dot
 
      val get_subgraph : V.t -> DotAttributes.subgraph option
        (** The box (if exists) which the vertex belongs to. Boxes with same
-	   names are not distinguished and so they should have the same
-	   attributes. *)
+           names are not distinguished and so they should have the same
+           attributes. *)
 
      val default_edge_attributes: t -> DotAttributes.edge list
      val edge_attributes: E.t -> DotAttributes.edge list
@@ -386,11 +431,11 @@ sig
 
   val fprint_graph: formatter -> X.t -> unit
     (** [fprint_graph ppf graph] pretty prints the graph [graph] in
-	the CGL language on the formatter [ppf]. *)
+        the CGL language on the formatter [ppf]. *)
 
   val output_graph: out_channel -> X.t -> unit
     (** [output_graph oc graph] pretty prints the graph [graph] in the dot
-	language on the channel [oc]. *)
+        language on the channel [oc]. *)
 
 end
 
@@ -410,13 +455,13 @@ module NeatoAttributes : sig
     | `Start of int
         (** Seed for random number generator. *)
     | `Overlap of bool
-	(** Default value is [true]. *)
+        (** Default value is [true]. *)
     | `Spline of bool
-	(** [true] makes edge splines if nodes don't overlap.
-	    Default value is [false]. *)
+        (** [true] makes edge splines if nodes don't overlap.
+            Default value is [false]. *)
     | `Sep of float
-	(** Edge spline separation factor from nodes.  Default value
-	    is [0.0]. *)
+        (** Edge spline separation factor from nodes.  Default value
+            is [0.0]. *)
     ]
 
   (** Attributes of nodes.  They include all common node attributes and
@@ -445,6 +490,7 @@ module NeatoAttributes : sig
   type subgraph = {
     sg_name : string;
     sg_attributes : vertex list;
+    sg_parent : string option;
   }
 
 end
@@ -471,8 +517,8 @@ module Neato
 
      val get_subgraph : V.t -> NeatoAttributes.subgraph option
        (** The box (if exists) which the vertex belongs to. Boxes with same
-	   names are not distinguished and so they should have the same
-	   attributes. *)
+           names are not distinguished and so they should have the same
+           attributes. *)
 
      val default_edge_attributes: t -> NeatoAttributes.edge list
      val edge_attributes: E.t -> NeatoAttributes.edge list
@@ -482,20 +528,20 @@ sig
 
   val set_command: string -> unit
     (** Several functions provided by this module run the external program
-	{i neato}.  By default, this command is supposed to be in the default
-	path and is invoked by {i neato}.  The function
-	[set_command] allows to set an alternative path at run time. *)
+        {i neato}.  By default, this command is supposed to be in the default
+        path and is invoked by {i neato}.  The function
+        [set_command] allows to set an alternative path at run time. *)
 
   exception Error of string
   val handle_error: ('a -> 'b) -> 'a -> 'b
 
   val fprint_graph: formatter -> X.t -> unit
     (** [fprint_graph ppf graph] pretty prints the graph [graph] in
-	the CGL language on the formatter [ppf]. *)
+        the CGL language on the formatter [ppf]. *)
 
   val output_graph: out_channel -> X.t -> unit
     (** [output_graph oc graph] pretty prints the graph [graph] in the dot
-	language on the channel [oc]. *)
+        language on the channel [oc]. *)
 
 end
 
