@@ -15,8 +15,6 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(* $Id: persistent.ml,v 1.18 2005-01-17 15:22:03 signoles Exp $ *)
-
 open Sig
 open Util
 open Blocks
@@ -24,7 +22,7 @@ open Blocks
 module type S = sig
 
   (** Persistent Unlabeled Graphs *)
-  module Concrete (V: COMPARABLE) : 
+  module Concrete (V: COMPARABLE) :
     Sig.P with type V.t = V.t and type V.label = V.t and type E.t = V.t * V.t
 	  and type E.label = unit
 
@@ -51,11 +49,11 @@ type 'a abstract_vertex = { tag : int; label : 'a }
 module AbstractVertex(V: sig type t end) = struct
   type label = V.t
   type t = label abstract_vertex
-  let compare x y = Pervasives.compare x.tag y.tag 
+  let compare x y = Pervasives.compare x.tag y.tag
   let hash x = x.tag
   let equal x y = x.tag = y.tag
   let label x = x.label
-  let create l = 
+  let create l =
     if !cpt_vertex = first_value_for_cpt_vertex - 1 then
       invalid_arg "Too much vertices";
     incr cpt_vertex;
@@ -64,7 +62,7 @@ end
 
 module Digraph = struct
 
-  module Concrete(V:COMPARABLE) = struct 
+  module Concrete(V:COMPARABLE) = struct
     include P.Digraph.Concrete(V)
     let remove_vertex g v =
       if HM.mem v g then
@@ -85,27 +83,75 @@ module Digraph = struct
 	g
   end
 
+  module ConcreteBidirectional(V: COMPARABLE) = struct
+    include P.Digraph.ConcreteBidirectional(V)
+    let remove_vertex g v =
+      if HM.mem v g then
+	let remove v = S.filter (fun v' -> not (V.equal v v')) in
+	let g =
+	  fold_pred
+	    (fun v' acc ->
+	       let in_set, out_set = HM.find v' acc in
+	       HM.add v' (in_set, remove v out_set) acc)
+	    g v g
+	in
+	let g =
+	  fold_succ
+	    (fun v' acc ->
+	       let in_set, out_set = HM.find v' acc in
+	       HM.add v' (remove v in_set, out_set) acc)
+	    g v g
+	in
+	HM.remove v g
+      else
+	g
+  end
+
+  module ConcreteBidirectionalLabeled(V:COMPARABLE)(E:ORDERED_TYPE_DFT) = struct
+    include P.Digraph.ConcreteBidirectionalLabeled(V)(E)
+    let remove_vertex (g:t) (v:vertex) =
+      if HM.mem v g then
+	let remove v = S.filter (fun (v', _) -> not (V.equal v v')) in
+	let g =
+	  fold_pred
+	    (fun v' acc ->
+	       let in_set, out_set = HM.find v' acc in
+	       HM.add v' (in_set, remove v out_set) acc)
+	    g v g
+	in
+	let g =
+	  fold_succ
+	    (fun v' acc ->
+	       let in_set, out_set = HM.find v' acc in
+	       HM.add v' (remove v in_set, out_set) acc)
+	    g v g
+	in
+	HM.remove v g
+      else
+	g
+  end
+
   module Abstract(V: sig type t end) = struct
 
     include P.Digraph.Abstract(AbstractVertex(V))
 
     let empty = { edges = G.empty; size = 0 }
 
-    let add_vertex g v = 
-      if mem_vertex g v then 
-	g 
+    let add_vertex g v =
+      if mem_vertex g v then
+	g
       else
-	{ edges = G.unsafe_add_vertex g.edges v; 
+	{ edges = G.unsafe_add_vertex g.edges v;
 	  size = Pervasives.succ g.size }
 
-    let add_edge g v1 v2 = 
+    let add_edge g v1 v2 =
       let g = add_vertex g v1 in
       let g = add_vertex g v2 in
       { g with edges = G.unsafe_add_edge g.edges v1 v2 }
 
     let add_edge_e g (v1, v2) = add_edge g v1 v2
 
-    let remove_vertex g v = 
+    let remove_vertex g v =
       if HM.mem v g.edges then
 	let e = HM.remove v g.edges in
 	let e = HM.fold (fun k s g -> HM.add k (S.remove v s) g) e HM.empty in
@@ -124,14 +170,14 @@ module Digraph = struct
 
     let empty = { edges = G.empty; size = 0 }
 
-    let add_vertex g v = 
-      if mem_vertex g v then 
-	g 
+    let add_vertex g v =
+      if mem_vertex g v then
+	g
       else
-	{ edges = G.unsafe_add_vertex g.edges v; 
+	{ edges = G.unsafe_add_vertex g.edges v;
 	  size = Pervasives.succ g.size }
 
-    let add_edge_e g (v1, l, v2) = 
+    let add_edge_e g (v1, l, v2) =
       let g = add_vertex g v1 in
       let g = add_vertex g v2 in
       { g with edges = G.unsafe_add_edge g.edges v1 (v2, l) }
@@ -141,12 +187,12 @@ module Digraph = struct
     let remove_vertex g v =
       if HM.mem v g.edges then
 	let remove v s =
-	  S.fold 
+	  S.fold
 	    (fun (v2, _ as e) s -> if not (V.equal v v2) then S.add e s else s)
 	    s S.empty
 	in
 	let edges = HM.remove v g.edges in
-	{ edges = 
+	{ edges =
 	    HM.fold (fun k s g -> HM.add k (remove v s) g) edges HM.empty;
 	  size = Pervasives.pred g.size }
       else
@@ -171,7 +217,7 @@ module Graph = struct
 
     (* Redefine the [add_edge] and [remove_edge] operations *)
 
-    let add_edge g v1 v2 = 
+    let add_edge g v1 v2 =
       let g = G.add_edge g v1 v2 in
       assert (G.HM.mem v1 g && G.HM.mem v2 g);
       G.unsafe_add_edge g v2 v1
@@ -189,9 +235,9 @@ module Graph = struct
 
   module ConcreteLabeled(V: COMPARABLE)(Edge: ORDERED_TYPE_DFT) = struct
 
-    module G = struct 
-      include Digraph.ConcreteLabeled(V)(Edge) 
-      type return = t 
+    module G = struct
+      include Digraph.ConcreteLabeled(V)(Edge)
+      type return = t
     end
     include Graph(G)
 
@@ -200,7 +246,7 @@ module Graph = struct
 
     (* Redefine the [add_edge] and [remove_edge] operations *)
 
-    let add_edge_e g (v1, l, v2 as e) = 
+    let add_edge_e g (v1, l, v2 as e) =
       let g = G.add_edge_e g e in
       assert (G.HM.mem v1 g && G.HM.mem v2 g);
       G.unsafe_add_edge g v2 (v1, l)
@@ -229,7 +275,7 @@ module Graph = struct
 
     (* Redefine the [add_edge] and [remove_edge] operations *)
 
-    let add_edge g v1 v2 = 
+    let add_edge g v1 v2 =
       let g = G.add_edge g v1 v2 in
       assert (G.HM.mem v1 g.G.edges && G.HM.mem v2 g.G.edges);
       { g with G.edges = G.unsafe_add_edge g.G.edges v2 v1 }
@@ -247,9 +293,9 @@ module Graph = struct
 
   module AbstractLabeled (V: sig type t end)(Edge: ORDERED_TYPE_DFT) = struct
 
-    module G = struct 
-      include Digraph.AbstractLabeled(V)(Edge) 
-      type return = t 
+    module G = struct
+      include Digraph.AbstractLabeled(V)(Edge)
+      type return = t
     end
     include Graph(G)
 
@@ -258,7 +304,7 @@ module Graph = struct
 
     (* Redefine the [add_edge] and [remove_edge] operations *)
 
-    let add_edge_e g (v1, l, v2 as e) = 
+    let add_edge_e g (v1, l, v2 as e) =
       let g = G.add_edge_e g e in
       assert (G.HM.mem v1 g.G.edges && G.HM.mem v2 g.G.edges);
       { g with G.edges = G.unsafe_add_edge g.G.edges v2 (v1, l) }
@@ -278,3 +324,9 @@ module Graph = struct
   end
 
 end
+
+(*
+Local Variables:
+compile-command: "make -C .."
+End:
+*)
