@@ -18,16 +18,20 @@
 (* $Id: rand.ml,v 1.18 2005-03-31 13:32:51 filliatr Exp $ *)
 
 module type S = sig
-  type graph 
+  type graph
   type vertex
   type edge_label
   val graph : ?loops:bool -> v:int -> e:int -> unit -> graph
-  val labeled : 
-    (vertex -> vertex -> edge_label) -> 
+  val labeled :
+    (vertex -> vertex -> edge_label) ->
     ?loops:bool -> v:int -> e:int -> unit -> graph
   (* DEBUG *)
   val random_few_edges : loops:bool -> v:int -> e:int -> graph
   val random_many_edges : loops:bool -> v:int -> e:int -> graph
+  val gnp : ?loops:bool -> v:int -> prob:float -> unit -> graph
+  val gnp_labeled :
+    (vertex -> vertex -> edge_label) ->
+    ?loops:bool -> v:int -> prob:float -> unit -> graph
 end
 
 module Make(B : Builder.INT) = struct
@@ -48,7 +52,7 @@ module Make(B : Builder.INT) = struct
     if of_int e > max_e then invalid_arg "random: too many edges";
     max_e
 
-  let fold_for i0 i1 f =
+   let fold_for i0 i1 f =
     let rec loop i v = if i > i1 then v else loop (i + 1) (f v i) in
     loop i0
 
@@ -60,7 +64,7 @@ module Make(B : Builder.INT) = struct
     let rec random_edge g =
       let i = Random.int v in
       let j = Random.int v in
-      if (i = j && not loops) || G.mem_edge g a.(i) a.(j) then 
+      if (i = j && not loops) || G.mem_edge g a.(i) a.(j) then
 	random_edge g
       else
 	add_edge g a.(i) a.(j)
@@ -74,12 +78,12 @@ module Make(B : Builder.INT) = struct
     let a = Array.init v G.V.create in
     let g = Array.fold_left add_vertex (empty ()) a in
     let rec add_edges i j max nb g =
-      assert 
+      assert
 	(max >= 0L &&
-         max_e = 
-	   add max (add (mul (of_int i) v64) 
-           (of_int 
-  	    (j - 
+         max_e =
+	   add max (add (mul (of_int i) v64)
+           (of_int
+  	    (j -
 	    (match G.is_directed, loops with
 	       | true, true -> 0
 	       | true, false -> if j > i then i + 1 else i
@@ -88,11 +92,11 @@ module Make(B : Builder.INT) = struct
       if nb = 0 then
 	g
       else
-	let add_edges = 
+	let add_edges =
 	  let i, j = if j = v - 1 then i + 1, 0 else i, j + 1 in
-	  add_edges i j 
+	  add_edges i j
 	in
-	if (i = j && not loops) || (not G.is_directed && i > j) then 
+	if (i = j && not loops) || (not G.is_directed && i > j) then
 	  add_edges max nb g
 	else
 	  let add_edges = add_edges (pred max) in
@@ -103,21 +107,45 @@ module Make(B : Builder.INT) = struct
     in
     add_edges 0 0 max_e e g
 
-  let random ~loops ~v ~e = 
+  let random ~loops ~v ~e =
     let r = float e /. (float v *. float v) in
     (if r < 0.4 then random_few_edges else random_many_edges) ~loops ~v ~e
 
   let graph ?(loops=false) ~v ~e () = random B.add_edge ~loops ~v ~e
 
-  let labeled f ?(loops=false) ~v ~e () = 
-    random 
-      (fun g v1 v2 -> B.add_edge_e g (G.E.create v1 (f v1 v2) v2)) 
+  let labeled f ?(loops=false) ~v ~e () =
+    random
+      (fun g v1 v2 -> B.add_edge_e g (G.E.create v1 (f v1 v2) v2))
       ~loops ~v ~e
 
   (* DEBUG *)
   let random_few_edges = random_few_edges B.add_edge
   let random_many_edges = random_many_edges B.add_edge
 
+  (** G(n,p) graphs
+      See https://en.wikipedia.org/wiki/Random_graph *)
+
+
+  let gnp_generic add_edge ?(loops=false) ~v ~prob () =
+    if not (0.0 <= prob && prob <= 1.0) then invalid_arg "gnp";
+    let vertices = Array.init v (fun i -> B.G.V.create i) in
+    let g = Array.fold_left B.add_vertex (B.empty ()) vertices in
+    let g = ref g in
+    for i = 0 to v-1 do
+      for j = 0 to (if G.is_directed then v-1 else i) do
+	if (loops || j <> i) && (prob = 1.0 || Random.float 1.0 < prob) then
+	  g := add_edge !g vertices.(i) vertices.(j)
+      done
+    done;
+    !g
+
+  let gnp ?(loops=false) ~v ~prob () =
+    gnp_generic B.add_edge ~loops ~v ~prob ()
+
+  let gnp_labeled f ?(loops=false) ~v ~prob () =
+    gnp_generic (fun g v1 v2 -> B.add_edge_e g (G.E.create v1 (f v1 v2) v2))
+      ~loops ~v ~prob ()
+
 end
 
 module P (G : Sig.P with type V.label = int) = Make(Builder.P(G))
@@ -130,8 +158,8 @@ module I (G : Sig.I with type V.label = int) = Make(Builder.I(G))
 module Planar = struct
 
   module type S = sig
-    type graph 
-    val graph : 
+    type graph
+    val graph :
       ?loops:bool -> xrange:int*int -> yrange:int*int ->
 	prob:float -> int -> graph
   end
@@ -145,10 +173,10 @@ module Planar = struct
 
     module Point = struct
       type point = V.t
-      let ccw v1 v2 v3 = 
+      let ccw v1 v2 v3 =
 	Delaunay.IntPoints.ccw (V.label v1) (V.label v2) (V.label v3)
       let in_circle v1 v2 v3 v4 =
-	Delaunay.IntPoints.in_circle 
+	Delaunay.IntPoints.in_circle
 	  (V.label v1) (V.label v2) (V.label v3) (V.label v4)
       let distance v1 v2 =
 	let x1,y1 = V.label v1 in
@@ -171,37 +199,43 @@ module Planar = struct
       let t = Triangulation.triangulate vertices in
       (* a graph with [v] vertices and random loops if any *)
       let g = Array.fold_left B.add_vertex (B.empty ()) vertices in
-      let g = 
-	if loops then 
-	  Array.fold_left 
-	    (fun g v -> 
-	       if Random.float 1.0 < prob then 
+      let g =
+	if loops then
+	  Array.fold_left
+	    (fun g v ->
+	       if Random.float 1.0 < prob then
 		 g
 	       else
 		 let e = E.create v 0 v in B.add_edge_e g e)
 	    g vertices
-	else 
+	else
 	  g
       in
       (* we keep some edges from the triangulation according to [prob] *)
       let add_edge v1 v2 g =
-	if Random.float 1.0 < prob then 
+	if Random.float 1.0 < prob then
 	  g
 	else
 	  let e = E.create v1 (Point.distance v1 v2) v2 in B.add_edge_e g e
       in
       Triangulation.fold
-	(fun v1 v2 g -> 
+	(fun v1 v2 g ->
 	   let g = add_edge v1 v2 g in
 	   if is_directed then add_edge v2 v1 g else g)
 	t g
 
   end
 
-  module P (G : Sig.P with type V.label = int * int and type E.label = int) = 
+  module P (G : Sig.P with type V.label = int * int and type E.label = int) =
     Make(Builder.P(G))
 
-  module I (G : Sig.I with type V.label = int * int and type E.label = int) = 
+  module I (G : Sig.I with type V.label = int * int and type E.label = int) =
     Make(Builder.I(G))
 
 end
+(*
+  Local Variables:
+  compile-command: "make -C .. src/rand.cmo"
+  End:
+
+*)
